function [varxmy,varx,vary] = varianza_suma_dos_va(xpart,xval,ypart,yval)
%  [varxmy,varx,vary] = varianza_suma_dos_va(xpart,xval,ypart,yval)
%  Dadas dos var aleat (X,Y) en forma particional, hallar var(X+Y)
%  Las variables estan definidas sobre el espacio muestral [0,1],
%     con una distribucion uniforme
%  xpart: vector (fila o columna) de puntos que definen la particion de X
%         1er elemento: 0, ultimo elemento: 1, orden ascendiente
%  ypart: vector (fila o columna) de puntos que definen la particion de Y
%         1er elemento: 0, ultimo elemento: 1, orden ascendiente
%  xval:  valores para los distintos elementos de la particion de X
%  yval:  valores para los distintos elementos de la particion de Y
%  varxmy: varianza de la suma X + Y
%  varx:  varianza de X
%  vary:  varianza de Y
% 
%  Ejemplo:  xpart = [0 1/4 1/3 1/2 3/4 1]  xval =  [10  15  20  15  10]
%            ypart = [0 1/3 5/6 1]  yval = [3 2 3]
%  El output es: varx = 13.889, vary = 0.25000, varxmy = 11.639

%% Funcion de Matlab/Octave -- Ricard Torres, 2015 (sin acentos)

% En primer lugar, comprobamos que los argumentos sean correctos
if ( nargin ~= 4 )
   mostrar_uso() ;
   error('La funcion debe tener cuatro argumentos') ;
end

if ~ and( isvector(xpart), and(isvector(xval), and(isvector(ypart), ...
     isvector(yval) ) ) )
   mostrar_uso() ;
   error('Los argumentos de la funcion deben de ser vectores') ;
end

lpx = length(xpart); lvx = length(xval);
lpy = length(ypart); lvy = length(yval);

if ~ and( ( lvx == lpx - 1 ),  ( lvy == lpy - 1 ) ) 
   mostrar_uso() ;
   error('Debe de haber un valor para cada elemento de la particion') ;
end

if ~ and( (xpart(1) == 0), and( (xpart(end) == 1), and( (ypart(1) == 0), ...
    (ypart(end) == 1) ) ) ) 
   mostrar_uso() ;
   error('Cada particion debe empezar con 0 y acabar con 1') ;
end

is_ascending = true;
for i = 2 : lpx
    if xpart(i) <= xpart(i-1), is_ascending = false; end
end
for i = 2 : lpy
    if ypart(i) <= ypart(i-1), is_ascending = false; end
end

if ~ is_ascending
   mostrar_uso() ;
   error('El vector que define cada particion debe tener orden ascendiente') ;
end

% Ahora desarrollamos los calculos propiamente dichos
[ distxy, valx, valy ] = distrib_dos_va(xpart,xval,ypart,yval);
distx = sum(distxy');
disty = sum(distxy);
mx = dot(distx, valx);
my = dot(disty, valy);
xc = valx - mx * ones(length(valx),1);
yc = valy - my * ones(1,length(valy));
varx = dot( distx, xc.^2 );
vary = dot( disty, yc.^2 );
covarxy = covarianza_dos_va(xpart,xval,ypart,yval);
varxmy = varx + vary + 2 * covarxy;

end

% Mensaje de ayuda en caso de error
function mostrar_uso()
fprintf('Uso: [varxmy,varx,vary] = varianza_suma_dos_va(xpart,xval,ypart,yval)\n');
fprintf('Dadas dos var aleat (X,Y) en forma particional, ');
fprintf('hallar var(X+Y)\n');
fprintf('Las variables estan definidas sobre el espacio muestral [0,1],\n');
fprintf('  con una distribucion uniforme\n');
fprintf('xpart: vector (fila o columna) de puntos que definen la particion de X\n');
fprintf('    1er elemento: 0, ultimo elemento: 1, orden ascendiente\n');
fprintf('ypart: vector (fila o columna) de puntos que definen la particion de Y\n');
fprintf('    1er elemento: 0, ultimo elemento: 1, orden ascendiente\n');
fprintf('xval:  valores para los distintos elementos ');
fprintf(' de la particion de X\n');
fprintf('yval:  valores para los distintos elementos ');
fprintf(' de la particion de Y\n');
fprintf('varxmy: varianza de la suma X + Y\n');
fprintf('varx: varianza de X\n');
fprintf('vary: varianza de Y\n');
fprintf('\nEjemplo:  xpart = [0 1/4 1/3 1/2 3/4 1] xval =  [10  15  20  15  10]\n');
fprintf('            ypart = [0 1/3 5/6 1]  yval = [3 2 3]\n');
fprintf('El output es: varx = 13.889, vary = 0.25000, varxmy = 11.639\n');
end

